use argh::FromArgs;

use crate::Error;
use crate::config::Config;

pub mod add;
pub use add::AddAction;
pub mod get;
pub use get::GetAction;
pub mod hash;
pub use hash::HashAction;
pub mod list;
pub use list::ListAction;
pub mod magnet;
pub use magnet::MagnetAction;
pub mod name;
pub use name::NameAction;

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand)]
pub enum Action {
    Add(AddAction),
    List(ListAction),
    Get(GetAction),
    Name(NameAction),
    Hash(HashAction),
    Magnet(MagnetAction),
}

pub trait ActionExec {
    fn exec(&self, config: &Config) -> Result<(), Error>;
}

impl Action {
    pub fn exec(&self, config: &Config) -> Result<(), Error> {
        match self {
            Action::Add(cmd) => {
                cmd.exec(&config)
            }, Action::List(cmd) => {
                cmd.exec(&config)
            }, Action::Get(cmd) => {
                cmd.exec(&config)
            }, Action::Hash(cmd) => {
                cmd.exec(&config)
            }, Action::Name(cmd) => {
                cmd.exec(&config)
            }, Action::Magnet(cmd) => {
                cmd.exec(&config)
            }
        }
 
    }
}
