use snafu::prelude::*;
//use snafu::*;

use std::path::PathBuf;

use crate::config::ConfigError;

#[derive(Debug, Snafu)]
#[snafu(visibility(pub))]
/// The possible error cases
pub enum Error {
    #[snafu(display("{}\nConfiguration file reading or parsing error (see above)", source))]
    Config { source: ConfigError },
    #[snafu(display("{}\nTorrent/magnet reading or parsing error", source))]
    Imdl { source: imdl::error::Error },
    #[snafu(display("{}\nqBittorrent API communication error", source))]
    Api { source: std::io::Error },
    #[snafu(display("{}\nInternal qBittorrent API error", source))]
    InternalApi { source: qbittorrent_web_api::api_impl::Error },
    #[snafu(display("Other error:\n{}", message))]
    Message { message: String },
    // New and better error messages
    #[snafu(display("Failed to reach qBittorrent API:\n{}", source))]
    FailedToReachAPI { source: qbittorrent_web_api::api_impl::Error },
    #[snafu(display("Failed to login to qBittorrent API with user: {}", user))]
    FailedLogin { user: String },
    #[snafu(display("Invalid torrent: {}", torrent.display()))]
    InvalidTorrent { torrent: PathBuf, source: imdl::error::Error },
    #[snafu(display("Invalid magnet: {}", magnet))]
    InvalidMagnet { magnet: String, source: imdl::magnet_link_parse_error::MagnetLinkParseError },
    #[snafu(display("Other torrent error inside imdl library with file {}:\n{}", torrent.display(), source))]
    OtherTorrent { torrent: PathBuf, source: imdl::error::Error },
    #[snafu(display("The magnet link contains no name: {}", magnet))]
    EmptyNameMagnet { magnet: String },
    #[snafu(display("Failed to read file {} because of underlying IO error:\n{}", path.display(), source))]
    FailedToReadFile { path: PathBuf, source: std::io::Error },
}

impl Error {
  pub fn message(message: impl Into<String>) -> Error {
    Self::Message {
      message: message.into(),
    }
  }
}
